# Examples demonstrating the use of the formalism described by Klatt & Polerecky (2015) 
# doi: 10.3389/fmicb.2015.00484

# Calculate CO2:TEA ratio as a function of x, assuming that the efficiency is (1) independent of x or (2) linearly
# decreasing from a maximum reached at x=0 to a minimum reached at x=1.
# Calculate for aerobic sulfide oxidation at standard biochemical conditions as well as at reactant concentrations of 1mM.

# to load the functions
source('SOX_functions.R')

# Assume that the efficiency is independent of x

# Beggiatoa str. MS-81-6
x <- 0; y <- 0.825
(e1 <- aerob_efficiency(x,y, Sred="H2S", approach=2, Cfix="Calvin")$eff)
S2O <- seq(from=0.62, to=3.5, length.out=100)
stoich1 <- aerob_stoichiometry(S2O, eff_fixed=e1, Sred="H2S", approach=2, Cfix="Calvin")
(e2 <- aerob_efficiency(x,y, Sred="H2S", approach=2, Cfix="Calvin", 
                        S=1e-3, O2=1e-3, CO2=1e-3, NAD=1e-3, NADH=1e-3, pH=7, sulfate=1e-3)$eff)
S2O <- seq(from=0.62, to=2.5, length.out=100)
stoich2 <- aerob_stoichiometry(S2O, eff_fixed=e2, Sred="H2S", approach=2, Cfix="Calvin", 
                               S=1e-3, O2=1e-3, CO2=1e-3, NAD=1e-3, NADH=1e-3, pH=7, sulfate=1e-3)

# Beggiatoa str. MS-81-1c
x <- 0.05; y <- 0.641
(e3 <- aerob_efficiency(x,y, Sred="H2S", approach=2, Cfix="Calvin")$eff)
S2O <- seq(from=0.62, to=3.5, length.out=100)
stoich3 <- aerob_stoichiometry(S2O, eff_fixed=e3, Sred="H2S", approach=2, Cfix="Calvin")
(e4 <- aerob_efficiency(x,y, Sred="H2S", approach=2, Cfix="Calvin", 
                        S=1e-3, O2=1e-3, CO2=1e-3, NAD=1e-3, NADH=1e-3, pH=7, sulfate=1e-3)$eff)
S2O <- seq(from=0.62, to=3.5, length.out=100)
stoich4 <- aerob_stoichiometry(S2O, eff_fixed=e4, Sred="H2S", approach=2, Cfix="Calvin", 
                               S=1e-3, O2=1e-3, CO2=1e-3, NAD=1e-3, NADH=1e-3, pH=7, sulfate=1e-3)

# plot the CO2:O2 ratio vs. x
x11(type="nbcairo")
par(mfrow=c(1,1), mar=c(5,5,1,1))
plot(stoich1$x,stoich1$CO2/stoich1$O2,type="l",lwd=2,lty=1,col='black',
     xlim=c(0,1),
     ylim=c(0.0,0.9),
     xlab="x",ylab=expression(CO[2]:O[2]),cex.lab=1.4,cex.axis=1.4)
lines(stoich2$x,stoich2$CO2/stoich2$O2,lwd=2,lty=2,col='black')
lines(stoich3$x,stoich3$CO2/stoich3$O2,lwd=2,lty=1,col='gray50')
lines(stoich4$x,stoich4$CO2/stoich4$O2,lwd=2,lty=2,col='gray50')

# include legend
legend('topright',legend=c("std. conditions","1 mM, pH 7"),
		bty="n", pch=c(NA,NA), lty=c(1,2), 
		lwd=c(2,2),cex=1.4)

# incude annotation
text(x=0.01,y=0.62,"Beggiatoa str. MS-81-1c",col="gray50",adj=c(0,NA),cex=1.2)
text(x=0.01,y=0.25,"Beggiatoa str. MS-81-6",col="black",adj=c(0,NA),cex=1.2)
text(x=0.01,y=0.01,"Efficiency = constant",col="black",adj=c(0,NA),cex=1.2)

# export to EPS
dev.copy2eps(file='CO2-TEA-x-aerobic-A.eps')

# Assume that the efficiency changes linearly between e0 at x=0 and e1 at x=1

# Beggiatoa str. MS-81-6
x0 <- 0; y <- 0.825
# determine e0 from x=x0 and the corresponding y  
e0_1 <- aerob_efficiency(x0,y, Sred="H2S", approach=2, Cfix="Calvin")$eff
e0_2 <- aerob_efficiency(x0,y, Sred="H2S", approach=2, Cfix="Calvin", 
                         S=1e-3, O2=1e-3, CO2=1e-3, NAD=1e-3, NADH=1e-3, pH=7, sulfate=1e-3)$eff 
e1_1 <- 0
e1_2 <- 0
# define e as a linear interpolation between e0 and e1
x <- seq(from=0,to=0.999,length.out=10)
e1 <- (e1_1-e0_1)/(1-x0)*(x-x0) + e0_1
e2 <- (e1_2-e0_2)/(1-x0)*(x-x0) + e0_2
CO2_O2_ratio1 <- vector(length=length(e1))
CO2_O2_ratio2 <- vector(length=length(e2))
for(i in 1:length(x)) {  
  stoich1 <- aerob_stoichiometry_y(x=x[i], eff_fixed=e1[i], Sred="H2S", approach=2, Cfix="Calvin")
  stoich2 <- aerob_stoichiometry_y(x=x[i], eff_fixed=e2[i], Sred="H2S", approach=2, Cfix="Calvin", 
                                   S=1e-3, O2=1e-3, CO2=1e-3, NAD=1e-3, NADH=1e-3, pH=7, sulfate=1e-3)
  CO2_O2_ratio1[i] <- stoich1$CO2/stoich1$O2
  CO2_O2_ratio2[i] <- stoich2$CO2/stoich2$O2
}

# Beggiatoa str. MS-81-1c
x0 <- 0.05; y <- 0.641
# determine e0 from x=x0 and the corresponding y  
e0_3 <- aerob_efficiency(x0,y, Sred="H2S", approach=2, Cfix="Calvin")$eff
e0_4 <- aerob_efficiency(x0,y, Sred="H2S", approach=2, Cfix="Calvin", 
                         S=1e-3, O2=1e-3, CO2=1e-3, NAD=1e-3, NADH=1e-3, pH=7, sulfate=1e-3)$eff 
e1_3 <- 0
e1_4 <- 0
# define e as a linear interpolation between e0 and e1
e3 <- (e1_3-e0_3)/(1-x0)*(x-x0) + e0_3
e4 <- (e1_4-e0_4)/(1-x0)*(x-x0) + e0_4
CO2_O2_ratio3 <- vector(length=length(e3))
CO2_O2_ratio4 <- vector(length=length(e4))
for(i in 1:length(x)) {
  stoich3 <- aerob_stoichiometry_y(x=x[i], eff_fixed=e3[i], Sred="H2S", approach=2, Cfix="Calvin")
  stoich4 <- aerob_stoichiometry_y(x=x[i], eff_fixed=e4[i], Sred="H2S", approach=2, Cfix="Calvin", 
                                   S=1e-3, O2=1e-3, CO2=1e-3, NAD=1e-3, NADH=1e-3, pH=7, sulfate=1e-3)
  CO2_O2_ratio3[i] <- stoich3$CO2/stoich3$O2
  CO2_O2_ratio4[i] <- stoich4$CO2/stoich4$O2
}

# plot efficiency vs. x, to illustrate how it looks like
x11(type="nbcairo")
par(mfrow=c(1,1), mar=c(5,5,1,1))
plot(x,e1,type="l",lwd=2,lty=1,col='black',
     xlim=c(0,1),
     ylim=c(0.0,0.5),
     xlab="x",ylab=expression(epsilon[II]),cex.lab=1.4,cex.axis=1.4)
lines(x,e2,lwd=2,lty=2,col='black')
lines(x,e3,lwd=2,lty=1,col='gray50')
lines(x,e4,lwd=2,lty=2,col='gray50')
# include legend
legend('topright',legend=c("std. conditions","1 mM, pH 7"),
       bty="n", pch=c(NA,NA), lty=c(1,2), 
       lwd=c(2,2),cex=1.4)

# incude annotation
text(x=0.01,y=0.3,"Beggiatoa str. MS-81-1c",col="gray50",adj=c(0,NA),cex=1.2)
text(x=0.01,y=0.115,"Beggiatoa str. MS-81-6",col="black",adj=c(0,NA),cex=1.2)

# export to EPS
dev.copy2eps(file='efficiency-x-aerobic-B.eps')

# plot the CO2:O2 ratio vs. x, which is the result we want to see
x11(type="nbcairo")
par(mfrow=c(1,1), mar=c(5,5,1,1))
plot(x,CO2_O2_ratio1,type="l",lwd=2,lty=1,col='black',
     xlim=c(0,1),
     ylim=c(0.0,0.9),
     xlab="x",ylab=expression(CO[2]:O[2]),cex.lab=1.4,cex.axis=1.4)
lines(x,CO2_O2_ratio2,lwd=2,lty=2,col='black')
lines(x,CO2_O2_ratio3,lwd=2,lty=1,col='gray50')
lines(x,CO2_O2_ratio4,lwd=2,lty=2,col='gray50')

# include legend
legend('topright',legend=c("std. conditions","1 mM, pH 7"),
       bty="n", pch=c(NA,NA), lty=c(1,2), 
       lwd=c(2,2),cex=1.4)

# incude annotation
text(x=0.01,y=0.62,"Beggiatoa str. MS-81-1c",col="gray50",adj=c(0,NA),cex=1.2)
text(x=0.01,y=0.25,"Beggiatoa str. MS-81-6",col="black",adj=c(0,NA),cex=1.2)

# export to EPS
dev.copy2eps(file='CO2-TEA-x-aerobic-B.eps')
