# Examples demonstrating the use of the formalism described by Klatt & Polerecky (2015) 
# doi: 10.3389/fmicb.2015.00484

# Calculate CO2:TEA ratio as a function of x, assuming that the efficiency is (1) independent of x or (2) linearly
# decreasing from a maximum reached at x=0 to a minimum reached at x=1.
# Calculate for anaerobic thiosulfate oxidation coupled to denitrification at standard biochemical conditions as well as at reactant concentrations of 1mM.

# to load the functions
source('SOX_functions.R')

# Assume that the efficiency is independent of x

# Sulfurimonas denitrificans
x <- 0; y <- 0.902
(e1 <- denit_efficiency(x,y, Sred="S2O3", approach=2, Cfix="rTCA")$eff)
S2N <- seq(from=0.01, to=3.5, length.out=300)
stoich1 <- denit_stoichiometry(S2N, eff_fixed=e1, Sred="S2O3", approach=2, Cfix="rTCA")
(e2 <- denit_efficiency(x,y, Sred="S2O3", approach=2, Cfix="rTCA", 
                        S=1e-3, NO3=1e-3, NAD=1e-3, NADH=1e-3, Fdox=1e-3, Fdred=1e-3, FAD=1e-3, FADH2=1e-3, 
                        pH=7, N2=1e-3, CO2=1e-3, sulfate=1e-3)$eff)
S2N <- seq(from=0.01, to=3.5, length.out=300)
stoich2 <- denit_stoichiometry(S2N, eff_fixed=e2, Sred="S2O3", approach=2, Cfix="rTCA", 
                               S=1e-3, NO3=1e-3, NAD=1e-3, NADH=1e-3, Fdox=1e-3, Fdred=1e-3, FAD=1e-3, FADH2=1e-3, 
                               pH=7, N2=1e-3, CO2=1e-3, sulfate=1e-3)

# Sulfurimonas hongkongensis
x <- 0; y <- 0.852
(e3 <- denit_efficiency(x,y, Sred="S2O3", approach=2, Cfix="rTCA")$eff)
S2N <- seq(from=0.62, to=3.5, length.out=100)
stoich3 <- denit_stoichiometry(S2N, eff_fixed=e3, Sred="S2O3", approach=2, Cfix="rTCA")
(e4 <- denit_efficiency(x,y, Sred="S2O3", approach=2, Cfix="rTCA", 
                        S=1e-3, NO3=1e-3, NAD=1e-3, NADH=1e-3, Fdox=1e-3, Fdred=1e-3, FAD=1e-3, FADH2=1e-3, 
                        pH=7, N2=1e-3, CO2=1e-3, sulfate=1e-3)$eff)
S2N <- seq(from=0.62, to=3.5, length.out=100)
stoich4 <- denit_stoichiometry(S2N, eff_fixed=e4, Sred="S2O3", approach=2, Cfix="rTCA", 
                               S=1e-3, NO3=1e-3, NAD=1e-3, NADH=1e-3, Fdox=1e-3, Fdred=1e-3, FAD=1e-3, FADH2=1e-3, 
                               pH=7, N2=1e-3, CO2=1e-3, sulfate=1e-3)

# plot the CO2:NO3 ratio vs. x
x11(type="nbcairo")
par(mfrow=c(1,1), mar=c(5,5,1,1))
plot(stoich1$x,stoich1$CO2/stoich1$NO3,type="l",lwd=2,lty=1,col='black',
     xlim=c(0,1),
     ylim=c(0.0,0.5),
     xlab="x",ylab=expression(CO[2]:NO[3]),cex.lab=1.4,cex.axis=1.4)
lines(stoich2$x,stoich2$CO2/stoich2$NO3,lwd=2,lty=2,col='black')
lines(stoich3$x,stoich3$CO2/stoich3$NO3,lwd=2,lty=1,col='gray50')
lines(stoich4$x,stoich4$CO2/stoich4$NO3,lwd=2,lty=2,col='gray50')

# include legend
legend('topright',legend=c("std. conditions","1 mM, pH 7"),
       bty="n", pch=c(NA,NA), lty=c(1,2), 
       lwd=c(2,2),cex=1.4)

# incude annotation
text(x=0.01,y=0.115,"Sulfurimonas denitrificans",col="black",adj=c(0,NA),cex=1.2)
text(x=0.01,y=0.25,"Sulfurimonas hongkongensis",col="gray50",adj=c(0,NA),cex=1.2)
text(x=0.01,y=0.01,"Efficiency = constant",col="black",adj=c(0,NA),cex=1.2)

# export to EPS
dev.copy2eps(file='CO2-TEA-x-denit-A.eps')

# Assume that the efficiency changes linearly between e0 at x=0 and e1 at x=1
  
# Sulfurimonas denitrificans
x0 <- 0; y <- 0.902
# determine e0 from x=x0 and the corresponding y  
e0_1 <- denit_efficiency(x0,y, Sred="S2O3", approach=2, Cfix="rTCA")$eff
e0_2 <- denit_efficiency(x0,y, Sred="S2O3", approach=2, Cfix="rTCA", 
                         S=1e-3, NO3=1e-3, NAD=1e-3, NADH=1e-3, Fdox=1e-3, Fdred=1e-3, FAD=1e-3, FADH2=1e-3, 
                         pH=7, N2=1e-3, CO2=1e-3, sulfate=1e-3)$eff 
e1_1 <- 0
e1_2 <- 0
# define e as a linear interpolation between e0 and e1
x <- seq(from=0,to=0.999,length.out=30)
e1 <- (e1_1-e0_1)/(1-x0)*(x-x0) + e0_1
e2 <- (e1_2-e0_2)/(1-x0)*(x-x0) + e0_2
CO2_NO3_ratio1 <- vector(length=length(e1))
CO2_NO3_ratio2 <- vector(length=length(e2))
for(i in 1:length(x)) {  
  stoich1 <- denit_stoichiometry_y(x=x[i], eff_fixed=e1[i], Sred="S2O3", approach=2, Cfix="rTCA")
  stoich2 <- denit_stoichiometry_y(x=x[i], eff_fixed=e2[i], Sred="S2O3", approach=2, Cfix="rTCA", 
                                   S=1e-3, NO3=1e-3, NAD=1e-3, NADH=1e-3, Fdox=1e-3, Fdred=1e-3, FAD=1e-3, FADH2=1e-3, 
                                   pH=7, N2=1e-3, CO2=1e-3, sulfate=1e-3)
  CO2_NO3_ratio1[i] <- stoich1$CO2/stoich1$NO3
  CO2_NO3_ratio2[i] <- stoich2$CO2/stoich2$NO3
}
  
# Sulfurimonas hongkongensis
x0 <- 0; y <- 0.852
# determine e0 from x=x0 and the corresponding y  
e0_3 <- denit_efficiency(x0,y, Sred="S2O3", approach=2, Cfix="rTCA")$eff
e0_4 <- denit_efficiency(x0,y, Sred="S2O3", approach=2, Cfix="rTCA", 
                         S=1e-3, NO3=1e-3, NAD=1e-3, NADH=1e-3, Fdox=1e-3, Fdred=1e-3, FAD=1e-3, FADH2=1e-3, 
                         pH=7, N2=1e-3, CO2=1e-3, sulfate=1e-3)$eff 
e1_3 <- 0
e1_4 <- 0
# define e as a linear interpolation between e0 and e1
x <- seq(from=0,to=0.999,length.out=30)
e3 <- (e1_3-e0_3)/(1-x0)*(x-x0) + e0_3
e4 <- (e1_4-e0_4)/(1-x0)*(x-x0) + e0_4
CO2_NO3_ratio3 <- vector(length=length(e3))
CO2_NO3_ratio4 <- vector(length=length(e4))
for(i in 1:length(x)) {  
  stoich3 <- denit_stoichiometry_y(x=x[i], eff_fixed=e3[i], Sred="S2O3", approach=2, Cfix="rTCA")
  stoich4 <- denit_stoichiometry_y(x=x[i], eff_fixed=e4[i], Sred="S2O3", approach=2, Cfix="rTCA", 
                                   S=1e-3, NO3=1e-3, NAD=1e-3, NADH=1e-3, Fdox=1e-3, Fdred=1e-3, FAD=1e-3, FADH2=1e-3, 
                                   pH=7, N2=1e-3, CO2=1e-3, sulfate=1e-3)
  CO2_NO3_ratio3[i] <- stoich3$CO2/stoich3$NO3
  CO2_NO3_ratio4[i] <- stoich4$CO2/stoich4$NO3
}

# plot efficiency vs. x, to illustrate how it looks like
x11(type="nbcairo")
par(mfrow=c(1,1), mar=c(5,5,1,1))
plot(x,e1,type="l",lwd=2,lty=1,col='black',
     xlim=c(0,1),
     ylim=c(0.0,0.1),
     xlab="x",ylab=expression(epsilon[II]),cex.lab=1.4,cex.axis=1.4)
lines(x,e2,lwd=2,lty=2,col='black')
lines(x,e3,lwd=2,lty=1,col='gray50')
lines(x,e4,lwd=2,lty=2,col='gray50')
# include legend
legend('topright',legend=c("std. conditions","1 mM, pH 7"),
       bty="n", pch=c(NA,NA), lty=c(1,2), 
       lwd=c(2,2),cex=1.4)

# incude annotation
text(x=0.01,y=0.082,"Sulfurimonas hongkongensis",col="gray50",adj=c(0,NA),cex=1.2)
text(x=0.01,y=0.019,"Sulfurimonas denitrificans",col="black",adj=c(0,NA),cex=1.2)

# export to EPS
dev.copy2eps(file='efficiency-x-denit-B.eps')

# plot the CO2:NO3 ratio vs. x, which is the result we want to see
x11(type="nbcairo")
par(mfrow=c(1,1), mar=c(5,5,1,1))
plot(x,CO2_NO3_ratio1,type="l",lwd=2,lty=1,col='black',
     xlim=c(0,1),
     ylim=c(0.0,0.3),
     xlab="x",ylab=expression(CO[2]:NO[3]),cex.lab=1.4,cex.axis=1.4)
lines(x,CO2_NO3_ratio2,lwd=2,lty=2,col='black')
lines(x,CO2_NO3_ratio3,lwd=2,lty=1,col='gray50')
lines(x,CO2_NO3_ratio4,lwd=2,lty=2,col='gray50')

# include legend
legend('topright',legend=c("std. conditions","1 mM, pH 7"),
       bty="n", pch=c(NA,NA), lty=c(1,2), 
       lwd=c(2,2),cex=1.4)

# incude annotation
text(x=0.01,y=0.23,"Sulfurimonas hongkongensis",col="gray50",adj=c(0,NA),cex=1.2)
text(x=0.01,y=0.059,"Sulfurimonas denitrificans",col="black",adj=c(0,NA),cex=1.2)

# export to EPS
dev.copy2eps(file='CO2-TEA-x-denit-B.eps')
