# Functions implementing the formalism described by Klatt & Polerecky (2015) 
# doi: 10.3389/fmicb.2015.00484

# Gibbs free energy of formation at standard biochemical conditions, in kJ/mol
dG0 <- list(sulfate = -744.63, 
			water = -237.18, 
			protons = 0, 
			H2S = -27.87,	  # sulfide
			S2O3 = -522.5,	# thiosulfate
			O2 = 16.4, 		  # O2 dissolved in water
			NO3=-111.3,		  # nitrate
			N2 = 18.2, 		  # di-nitrogen
			Corg = -152.57, # CH2O assumed as fructose (C6H1206), the value is normalized per one C
	    CO2 = -386.02, 
			S0 = 0, 		    # zero-valent sulfur
			deltaNAD=60.99, # dGf^0(NADH)-dGf^0(NAD+)
			deltaFd=38.88,  # dGf^0(Fdred)-dGf^0(Fdox)
			deltaFAD=42.65, # dGf^0(FADH2)-dGf^0(FAD)
  		EATP = 41,      # Energy content of ATP
  		EATP_Calvin= 3*41, # dE(CO2 fix) in kJ per mole CO2 (3ATP per CO2)
  		EATP_rTCA = 5/3*41, # dE(CO2 fix) in kJ per mole CO2 (1.67ATP per CO2)
			dG_Calvin=69.67, # dG(CO2 fix) in kJ per mole CO2
			dG_rTCA=64.33)   # dG(CO2 fix) in kJ per mole CO2

# universal gas constant, in kJ/mol/K
R <- 0.00831451

#####################################################################
# Aerobic sulfur oxidation coupled to CO2 fixation
# Implemented for reduced sulfur compounds Sred="H2S" and Sred="S2O3"
#####################################################################

# stoichiometric coefficients and other parameters as a function of x and y
aerob_coeffs <- function(x=0, y=0.825, Sred="H2S") {
  if(Sred == "H2S") {
    dG0S <- dG0$H2S
    # energy generating reaction
    o2 <- 2-1.5*x; s0 <- x; sulfate <- 1-x; water1 <- x; protons1 <- 2*(1-x)
    # energy conserving reaction, traditional approach
    co2 <- 2-1.5*x; water2 <- -(2-2.5*x); corg <- co2; protons2 <- 2*(1-x)
    # energy conserving reaction, new approach
    nad <- 4-3*x; nad2 <- 2-1.5*x; fad <- 2/3-0.5*x; fd <- 8/3-2*x; 
	  water3 <- -(4-4*x); protons3 <- 6-5*x; protons4 <- 20/3-11/2*x
  } 
  else if(Sred == "S2O3") {
    dG0S <- dG0$S2O3
    # energy generating reaction
    o2 <- 2-1.5*x; s0 <- x; sulfate <- 2-x; water1 <- x-1; protons1 <- 2*(1-x)
    # energy conserving reaction, traditional approach
    co2 <- 2-1.5*x; water2 <- -(3-2.5*x); corg <- co2; protons2 <- 2*(1-x)
    # energy conserving reaction, new approach
    nad <- 4-3*x; nad2 <- 2-1.5*x; fad <- 2/3-0.5*x; fd <- 8/3-2*x; 
	  water3 <- -(5-4*x); protons3 <- 6-8*x; protons4 <- 20/3-11/2*x
  }
  # generalized coupled reaction
  no2 <- y*o2
  nco2 <- (1-y)*co2
  ncorg <- nco2
  nwater <- y*water1 + (1-y)*water2
  ns0 <- s0
  nsulfate <- sulfate
  nprotons <- y*protons1 + (1-y)*protons2
  return(list(dG0S=dG0S, o2=o2, s0=s0, sulfate=sulfate, water1=water1, protons1=protons1, 
      protons2=protons2, co2=co2, water2=water2, corg=corg, nad=nad,
			nad2=nad2, fad=fad, fd=fd, water3=water3, protons3=protons3, protons4=protons4, 
			no2=no2, nco2=nco2, ncorg=ncorg, 
			nwater=nwater, ns0=ns0, nsulfate=nsulfate, nprotons=nprotons))
}

# function for calculating the efficiency from known stoichiometry
# - stoichiometry is given by parameters x and y
# - calculation approach (traditional vs. new) is specified through input parameter "approach" 
#   (1=traditional, 2=new)
#   for the new approach the carbon fixation pathway will also need to be specified (Calvin or rTCA)
# - additional arguments include the actual concentrations (in mol/L) of the reactants involved
#   (sulfate, S=reduced sulfur compound, O2, CO2), temperature (in degC), pH, redox carriers
#   (NAD/NADH, FAD/FADH2, Fdox/Fdred) and the type of the reduced sulfur compound (Sred="H2S", "S2O3")
# - default values of the additional arguments correspond to standard biochemical conditions
#   (reactant concentrations 1M, Temperature 25 degC, pH 7)
aerob_efficiency <- function(x=0, y=0.825, Sred="H2S", approach=2, Cfix="Calvin", 
				sulfate=1, S=1, O2=1, CO2=1, Temp=25, pH=7, 
				NAD=1, NADH=1, FAD=1, FADH2=1, Fdred=1, Fdox=1){
  protons <- 10^(-pH)
  # fill in coefficients, depending on the kind of the reduced S compound
  coef<-aerob_coeffs(x=x, y=y, Sred=Sred)
  # quotients Q
  # energy generating reaction
  Q1 <- sulfate^coef$sulfate * protons^coef$protons1 * S^(-1) * O2^(-coef$o2)
  # energy conserving reaction, traditional approach
  Q2 <- sulfate^coef$sulfate * protons^coef$protons2 * S^(-1) * CO2^(-coef$co2)
  # energy conserving reaction, new approach
  Q3 <- sulfate^coef$sulfate * protons^coef$protons3 * NADH^coef$nad * S^(-1) * NAD^(-coef$nad)
  Q4 <- sulfate^coef$sulfate * protons^coef$protons4 * NADH^coef$nad2 * Fdred^coef$fd * 
    FADH2^coef$fad * S^(-1) * NAD^(-coef$nad2) * Fdox^(-coef$fd) * FAD^(-coef$fad)
  # Gibbs free energies, in kJ per mole Sred
  # O2 reduction
  dGr0_O2red <- coef$s0*dG0$S0 + coef$sulfate*dG0$sulfate + coef$water1*dG0$water + 
				coef$protons1*dG0$protons - 1*coef$dG0S - coef$o2*dG0$O2
  # CO2 reduction  
  if (approach==1) {
    dGr0_CO2fix <- coef$s0*dG0$S0 + coef$sulfate*dG0$sulfate + coef$corg*dG0$Corg + 
				           coef$protons2*dG0$protons + coef$water2*dG0$water - 
			   	         1*coef$dG0S - coef$co2*dG0$CO2 # kJ per mole Sred
    dGr0_RET <- 0
  } 
  else if (approach==2) { 
    if (Cfix=="Calvin") {
      dGr0_CO2fix <- coef$co2*dG0$dG_Calvin # kJ per mole Sred
      dGr0_RET <- coef$s0*dG0$S0 + coef$sulfate*dG0$sulfate + coef$water3*dG0$water +
                  coef$nad*dG0$deltaNAD + coef$protons3*dG0$protons - 1*coef$dG0S # kJ per mole Sred
	  } else if (Cfix=="rTCA") {
	    dGr0_CO2fix <- coef$co2*dG0$dG_rTCA # kJ per mole Sred
      dGr0_RET <- coef$s0*dG0$S0 + coef$sulfate*dG0$sulfate + coef$water3*dG0$water +
                  coef$nad2*dG0$deltaNAD + coef$fad*dG0$deltaFAD+ coef$fd*dG0$deltaFd + 
                  coef$protons4*dG0$protons - 1*coef$dG0S # kJ per mole Sred
	  }    
  }
  dGr0_CO2red <- dGr0_CO2fix + dGr0_RET # kJ per mole Sred
  # calculate T-correction and quotient Q
  if (approach==1) {Qx <- Q2}
  if (approach==2) { 
    if (Cfix=="Calvin") {Qx <- Q3}
    if (Cfix=="rTCA")   {Qx <- Q4}
  }
  corr_O2red <-  R*(273.15+Temp)*log(Q1)
  corr_CO2red <- R*(273.15+Temp)*log(Qx)
  Q <- Q1^y * Qx^(1-y)
  # apply T-correction
  dGr_O2red <- dGr0_O2red + corr_O2red
  dGr_CO2red <- dGr0_CO2red + corr_CO2red
  # finally calculate the energy conservation efficiencies
  eff <- -(1-y)*dGr_CO2red / ( y*dGr_O2red )
  # fill the other output values
  if (approach==1) {
	  eff_CO2 <- NA; alpha <- NA; eff_RET_min <- NA; eff_SO_min <- eff
  }
  else if (approach==2) {
	  if (Cfix=="Calvin") {eff_CO2 <- dG0$dG_Calvin/dG0$EATP_Calvin}
	  if (Cfix=="rTCA")   {eff_CO2 <- dG0$dG_rTCA/dG0$EATP_rTCA}
	  alpha <- dGr0_CO2fix/dGr_CO2red
    eff_RET_min <- (1-alpha)/(1/eff - alpha/eff_CO2)
    eff_SO_min <- eff*(alpha/eff_CO2 + (1-alpha))
  }
  # in addition to efficiencies (eff, eff_CO2, eff_RET_min, eff_SO_min), 
  # the function also returns the Gibbs free energies of the energy gaining
  # (dGr_O2red) and energy conserving (dGr_CO2red) parts of the reaction,
  # as well as the quotient Q and coefficient alpha
  return(list(eff=eff, eff_CO2=eff_CO2, eff_SO_min=eff_SO_min, eff_RET_min=eff_RET_min, 
              alpha=alpha, Q=Q, Q1=Q1, Qx=Qx,
              dGr_RET=dGr0_RET+corr_CO2red, dGr_CO2red=dGr_CO2red, dGr_O2red=dGr_O2red))
}

##############################################
# Find stoichiometric coefficients for a given
# efficiency and Stot:O2 ratio
##############################################

# Function for calculating the difference between a given efficiency (eff_fixed) and the efficiency 
# calculated from the stoichiometry parameter x and the Stot:O2 consumption ratio (input variable S2O).
# This is the function for which a root will need to be found in order to determine the stoichiometry
# for a given efficiency and Stot:O2 consumption ratio.
aerob_deff <- function(x=0, S2O=1, eff_fixed=0.05, approach=2, Cfix="Calvin", Sred="H2S", 
			sulfate=1, S=1, O2=1, CO2=1, Temp=25, pH=7, 
			NAD=1, NADH=1, FAD=1, FADH2=1, Fdred=1, Fdox=1){
  if (Sred=="H2S" | Sred=="S2O3") {
	  y <- 1/(S2O*(2-1.5*x))
  }
  e <- aerob_efficiency(x=x, y=y, approach=approach, Cfix=Cfix, Sred=Sred, 
		  sulfate=sulfate, S=S, O2=O2, CO2=CO2, Temp=Temp, pH=pH, 
		  NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox)$eff
  return(e-eff_fixed)
}

# Function for calculating the difference between a given efficiency (eff_fixed) and the efficiency 
# calculated from the stoichiometry parameters x and y.
# This is the function for which a root will need to be found in order to determine the stoichiometry
# for a given efficiency and x.
aerob_deff_y <- function(y=0.5, x=0, eff_fixed=0.05, approach=2, Cfix="Calvin", Sred="H2S", 
                       sulfate=1, S=1, O2=1, CO2=1, Temp=25, pH=7, 
                       NAD=1, NADH=1, FAD=1, FADH2=1, Fdred=1, Fdox=1){
  e <- aerob_efficiency(x=x, y=y, approach=approach, Cfix=Cfix, Sred=Sred, 
                        sulfate=sulfate, S=S, O2=O2, CO2=CO2, Temp=Temp, pH=pH, 
                        NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox)$eff
  return(e-eff_fixed)
}

# Function for calculating the stoichiometry coefficients (normalized to Sred=1)
# Input: one fixed value of efficiency (eff_fixed), a range of Stot:O2 ratios (S2O), reactant concentrations
aerob_stoichiometry <- function(S2O=1, eff_fixed=0.05, approach=2, Cfix="Calvin", Sred="H2S", 
					sulfate=1, S=1, O2=1, CO2=1, Temp=25, pH=7, 
					NAD=1, NADH=1, FAD=1, FADH2=1, Fdred=1, Fdox=1){
  int <- c(-0.1,1.1)
  #xint <- seq(min(int),max(int),length.out=100) # for debugging
  x <- vector(length=length(S2O))
  y <- vector(length=length(S2O))
  for(i in 1:length(S2O)) {
    s2o <- S2O[i]
    #plot(xint,aerob_deff(xint,S2O=s2o,efficiency=eff),type="l") # for debugging
    # find the root of aerob_deff, which effectively finds x for which the calculated efficiency equals eff_fixed
    # first check whether it should exist, to avoid error messages of the uniroot function
    if( aerob_deff(x=min(int), S2O=s2o, eff_fixed=eff_fixed, approach=approach, Cfix=Cfix, Sred=Sred, 
		    	sulfate=sulfate, S=S, O2=O2, CO2=CO2, Temp=Temp, pH=pH,
			    NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox) *
        aerob_deff(x=max(int), S2O=s2o, eff_fixed=eff_fixed, approach=approach, Cfix=Cfix, Sred=Sred, 
			    sulfate=sulfate, S=S, O2=O2, CO2=CO2, Temp=Temp, pH=pH, 
			    NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox) < 0 ) {
      r <- uniroot(aerob_deff, interval=int, S2O=s2o, eff_fixed=eff_fixed, approach=approach, Cfix=Cfix, Sred=Sred, 
			              sulfate=sulfate, S=S, O2=O2, CO2=CO2, Temp=Temp, pH=pH, 
			              NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox)
      if( r$root>=-1e-4 & r$root<=1.0001 ) { # store root if it is between 0 and 1 (+/- some tolerance)
        x[i] <- r$root
        #points(x[i],y=0) # for debugging
      } else {
        x[i] <- NA
      }
    } else {
      if( abs(aerob_deff(x=0,S2O=s2o, eff_fixed=eff_fixed, approach=approach, Cfix=Cfix, Sred=Sred, 
				        sulfate=sulfate, S=S, O2=O2, CO2=CO2, Temp=Temp, pH=pH, 
				        NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox)) < 1e-5 ) {
        x[i] <- 0
      } else {
		    x[i] <- NA
	    }
    }
    #readline("Press enter to continue") # for debugging
	  if (Sred=="H2S" | Sred=="S2O3") {
      y[i] <- 1/(s2o*(2-1.5*x[i])) # calculate y from the given s2o and from the found x
	  }
  }
  # select only numbers that are not NA
  i <- !is.na(x)
  x <- x[i]
  y <- y[i]
  S2O <- S2O[i]
  # calculate stoichiometric coefficients for all reactants in the generalized reaction, depending on the kind of Sred
  # note that the coefficient in front of Sred is 1
  coef<-aerob_coeffs(x=x, y=y, Sred=Sred)
  # display the stoichiometry of the reaction and the ceresponding energy gained and conserved
  # do this only if *one* value of the Sred:O2 ratio (S2O) was specified in the input
  if (length(S2O)==1)
  {
    cat(sprintf("Stoichiometry parameters:\tx = %.4f, y = %.4f\n",x,y))
    e <- aerob_efficiency(x=x, y=y, approach=approach, Cfix=Cfix, Sred=Sred, 
			sulfate=sulfate, S=S, O2=O2, CO2=CO2, Temp=Temp, pH=pH, 
			NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox)
	  cat(sprintf("Energy-gaining reaction:\n\t%.3f %s + %.3f O2 -> %.3f S0 + %.3f SO4^{2-} + %.3f H2O + %.3f H+\n",
                y,Sred,y*coef$o2,y*coef$s0,y*coef$sulfate,y*coef$water1,y*coef$protons1))
    cat(sprintf("dG(O2-reduction) = %.2f kJ/(mol H2S) = %.2f kJ/(mol O2)\n", e$dGr_O2red, e$dGr_O2red/coef$o2))
	  if (approach==1) {
	    cat(sprintf("Energy-conserving reaction:\n\t%.3f %s + %.3f CO2 -> %.3f S0 + %.3f SO4^{2-} + %.3f CH2O + %.3f H2O + %.3f H+\n",
                1-y,Sred,(1-y)*coef$co2,(1-y)*coef$s0,(1-y)*coef$sulfate,(1-y)*coef$co2,(1-y)*coef$water2,(1-y)*coef$protons2))
	  } else if(approach==2) {
		  if(Cfix=="Calvin") {
		    cat(sprintf("Energy-conserving reaction:\n\t%.3f %s + %.3f NAD -> %.3f S0 + %.3f SO4^{2-} + %.3f NADH + %.3f H2O + %.3f H+\n",
              1-y,Sred,(1-y)*coef$nad,(1-y)*coef$s0,(1-y)*coef$sulfate,(1-y)*coef$nad,(1-y)*coef$water3,(1-y)*coef$protons3))
		  }
      if (Cfix=="rTCA") {
		    cat(sprintf("Energy-conserving reaction:\n\t%.3f %s + %.3f NAD + %.3f Fe_ox + %.3f FAD-> %.3f S0 + %.3f SO4^{2-} + %.3f NADH + %.3f Fe_red + %.3f FADH2 + %.3f H2O + %.3f H+\n",
              1-y,Sred,(1-y)*coef$nad2,(1-y)*coef$fd,(1-y)*coef$fad,(1-y)*coef$s0,(1-y)*coef$sulfate,(1-y)*coef$nad2,(1-y)*coef$fd,(1-y)*coef$fad,(1-y)*coef$water3,(1-y)*coef$protons4))
		  }
  	}
    cat(sprintf("dG(CO2-reduction) = %.2f kJ/(mol H2S) = %.2f kJ/(mol O2)\n", e$dGr_CO2red, e$dGr_CO2red/coef$co2))     
    cat(sprintf("Coupled reaction:\n\t%s + %.3f O2 + %.3f CO2 -> %.3f S0 + %.3f SO4^{2-} + %.3f CH2O + %.3f H2O + %.3f H+\n",
                Sred,coef$no2,coef$nco2,coef$ns0,coef$nsulfate,coef$ncorg,coef$nwater,coef$nprotons))    
  }
  return(list(x=x, y=y, O2=coef$no2, CO2=coef$nco2, S0=coef$ns0, 
			sulfate=coef$nsulfate, Corg=coef$ncorg, water=coef$nwater, 
			protons=coef$nprotons, S2O=S2O))
}

# Function for calculating the stoichiometry coefficients (normalized to Sred=1)
# Input: one fixed value of efficiency (eff_fixed) and one value of x, reactant concentrations
aerob_stoichiometry_y <- function(x=0, eff_fixed=0.05, approach=2, Cfix="Calvin", Sred="H2S", 
                                sulfate=1, S=1, O2=1, CO2=1, Temp=25, pH=7, 
                                NAD=1, NADH=1, FAD=1, FADH2=1, Fdred=1, Fdox=1){
  int <- c(1e-4,1)
  # find the root of aerob_deff_y, which effectively finds y for which the calculated efficiency equals eff_fixed
  # first check whether it should exist, to avoid error messages of the uniroot function
  if( aerob_deff_y(y=min(int), x=x, eff_fixed=eff_fixed, approach=approach, Cfix=Cfix, Sred=Sred, 
                   sulfate=sulfate, S=S, O2=O2, CO2=CO2, Temp=Temp, pH=pH,
                   NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox) *
      aerob_deff_y(y=max(int), x=x, eff_fixed=eff_fixed, approach=approach, Cfix=Cfix, Sred=Sred, 
                   sulfate=sulfate, S=S, O2=O2, CO2=CO2, Temp=Temp, pH=pH, 
                   NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox) < 0 ) {
    r <- uniroot(aerob_deff_y, interval=int, x=x, eff_fixed=eff_fixed, approach=approach, Cfix=Cfix, Sred=Sred, 
                 sulfate=sulfate, S=S, O2=O2, CO2=CO2, Temp=Temp, pH=pH, 
                 NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox)
    if( r$root>=-1e-4 & r$root<=1.0001 ) { # store root if it is between 0 and 1 (+/- some tolerance)
      y <- r$root
    } else {
      y <- NA
    }
  } else {
    if( abs(aerob_deff_y(y=0,x=x, eff_fixed=eff_fixed, approach=approach, Cfix=Cfix, Sred=Sred, 
                       sulfate=sulfate, S=S, O2=O2, CO2=CO2, Temp=Temp, pH=pH, 
                       NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox)) < 1e-5 ) {
      y <- 0
    } else {
      y <- NA
    }
  }
  # calculate stoichiometric coefficients for all reactants in the generalized reaction, depending on the kind of Sred
  # note that the coefficient in front of Sred is 1
  coef<-aerob_coeffs(x=x, y=y, Sred=Sred)
  # display the stoichiometry of the reaction and the ceresponding energy gained and conserved
  # do this only if *one* value of the Sred:O2 ratio (S2O) was specified in the input
  if (!is.na(y))
  {
    cat(sprintf("Stoichiometry parameters:\tx = %.4f, y = %.4f\n",x,y))
    e <- aerob_efficiency(x=x, y=y, approach=approach, Cfix=Cfix, Sred=Sred, 
                          sulfate=sulfate, S=S, O2=O2, CO2=CO2, Temp=Temp, pH=pH, 
                          NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox)
    cat(sprintf("Energy-gaining reaction:\n\t%.3f %s + %.3f O2 -> %.3f S0 + %.3f SO4^{2-} + %.3f H2O + %.3f H+\n",
                y,Sred,y*coef$o2,y*coef$s0,y*coef$sulfate,y*coef$water1,y*coef$protons1))
    cat(sprintf("dG(O2-reduction) = %.2f kJ/(mol H2S) = %.2f kJ/(mol O2)\n", e$dGr_O2red, e$dGr_O2red/coef$o2))
    if (approach==1) {
      cat(sprintf("Energy-conserving reaction:\n\t%.3f %s + %.3f CO2 -> %.3f S0 + %.3f SO4^{2-} + %.3f CH2O + %.3f H2O + %.3f H+\n",
                  1-y,Sred,(1-y)*coef$co2,(1-y)*coef$s0,(1-y)*coef$sulfate,(1-y)*coef$co2,(1-y)*coef$water2,(1-y)*coef$protons2))
    } else if(approach==2) {
      if(Cfix=="Calvin") {
        cat(sprintf("Energy-conserving reaction:\n\t%.3f %s + %.3f NAD -> %.3f S0 + %.3f SO4^{2-} + %.3f NADH + %.3f H2O + %.3f H+\n",
                    1-y,Sred,(1-y)*coef$nad,(1-y)*coef$s0,(1-y)*coef$sulfate,(1-y)*coef$nad,(1-y)*coef$water3,(1-y)*coef$protons3))
      }
      if (Cfix=="rTCA") {
        cat(sprintf("Energy-conserving reaction:\n\t%.3f %s + %.3f NAD + %.3f Fe_ox + %.3f FAD-> %.3f S0 + %.3f SO4^{2-} + %.3f NADH + %.3f Fe_red + %.3f FADH2 + %.3f H2O + %.3f H+\n",
                    1-y,Sred,(1-y)*coef$nad2,(1-y)*coef$fd,(1-y)*coef$fad,(1-y)*coef$s0,(1-y)*coef$sulfate,(1-y)*coef$nad2,(1-y)*coef$fd,(1-y)*coef$fad,(1-y)*coef$water3,(1-y)*coef$protons4))
      }
    }
    cat(sprintf("dG(CO2-reduction) = %.2f kJ/(mol H2S) = %.2f kJ/(mol O2)\n", e$dGr_CO2red, e$dGr_CO2red/coef$co2))     
    cat(sprintf("Coupled reaction:\n\t%s + %.3f O2 + %.3f CO2 -> %.3f S0 + %.3f SO4^{2-} + %.3f CH2O + %.3f H2O + %.3f H+\n",
                Sred,coef$no2,coef$nco2,coef$ns0,coef$nsulfate,coef$ncorg,coef$nwater,coef$nprotons))    
  }
  return(list(x=x, y=y, O2=coef$no2, CO2=coef$nco2, S0=coef$ns0, 
              sulfate=coef$nsulfate, Corg=coef$ncorg, water=coef$nwater, 
              protons=coef$nprotons))
}

#####################################################################
# Sulfur oxidation coupled to denitrification and CO2 fixation
# Implemented for reduced sulfur compounds Sred="H2S" and Sred="S2O3"
#####################################################################

# stoichiometric coefficients and other parameters as a function of x and y
denit_coeffs <- function(x=0, y=0.825, Sred="H2S") {
  if(Sred == "H2S") {
    dG0S <- dG0$H2S
    # energy generating reaction
    no3 <- 1.6-1.2*x; s0 <- x; sulfate <- 1-x; water1 <- 0.8 + 0.4*x; n2 <- 0.8-0.6*x; protons1 <- 0.4+0.2*x
    # energy conserving reaction, traditional approach
    co2 <- 2-1.5*x; water2 <- -(2-2.5*x); corg <- co2; protons2 <- 2*(1-x)
    # energy conserving reaction, new approach
    nad <- 4-3*x; nad2 <- 2-1.5*x; fad <- 2/3-0.5*x; fd <- 8/3-2*x; 
	  water3 <- -(4-4*x); protons3 <- 6-5*x; protons4 <- 20/3-11/2*x
  } 
  else if(Sred == "S2O3") {
    dG0S <- dG0$S2O3
    # energy generating reaction
    no3 <- 1.6-1.2*x; s0 <- x; sulfate <- 2-x; water1 <- -0.2 + 0.4*x; n2 <- 0.8-0.6*x; protons1 <- 0.4-0.8*x
    # energy conserving reaction, traditional approach
    co2 <- 2-1.5*x; water2 <- -(3-2.5*x); corg <- co2; protons2 <- 2*(1-x)
    # energy conserving reaction, new approach
    nad <- 4-3*x; nad2 <- 2-1.5*x; fad <- 2/3-0.5*x; fd <- 8/3-2*x; 
	  water3 <- -(5-4*x); protons3 <- 6-8*x; protons4 <- 20/3-11/2*x
  }
  # generalized coupled reaction
  nno3 <- y*no3
  nn2 <- y*n2
  nco2 <- (1-y)*co2
  ncorg <- nco2
  nwater <- y*water1 + (1-y)*water2
  ns0 <- s0
  nsulfate <- sulfate
  nprotons <- y*protons1 + (1-y)*protons2
  return(list(dG0S=dG0S, no3=no3, s0=s0, sulfate=sulfate, water1=water1, n2=n2, protons1=protons1, 
      protons2=protons2, co2=co2, water2=water2, corg=corg, nad=nad,
			nad2=nad2, fad=fad, fd=fd, water3=water3, protons3=protons3, protons4=protons4, 
			nno3=nno3, nn2=nn2, nco2=nco2, ncorg=ncorg, 
			nwater=nwater, ns0=ns0, nsulfate=nsulfate, nprotons=nprotons))
}

# function for calculating the efficiency from known stoichiometry
# - stoichiometry is given by parameters x and y
# - calculation approach (traditional vs. new) is specified through input parameter "approach"
#   for the new approach the carbon fixation pathway will also need to be specified (Calvin or rTCA)
# - additional arguments include the actual concentrations (in mol/L) of the reactants involved
#   (sulfate, S=reduced sulfur compound, NO3, N2, CO2), temperature (in degC), pH, redox carriers
#   (NAD/NADH, FAD/FADH2, Fdox/Fdred) and the type of the reduced sulfur compound (Sred="H2S", "S2O3")
# - default values of the additional arguments correspond to standard biochemical conditions
#   (reactant concentrations 1M, Temperature 25 degC, pH 7)
denit_efficiency <- function(x=0, y=0.825, Sred="H2S", approach=2, Cfix="Calvin", 
				sulfate=1, S=1, NO3=1, N2=1, CO2=1, Temp=25, pH=7, 
				NAD=1, NADH=1, FAD=1, FADH2=1, Fdred=1, Fdox=1){
  protons <- 10^(-pH)
  # fill in coefficients, depending on the kind of the reduced S compound
  coef<-denit_coeffs(x=x, y=y, Sred=Sred)
  # quotients Q
  # energy generating reaction
  Q1 <- sulfate^coef$sulfate * protons^coef$protons1 * N2^coef$n2 * S^(-1) * NO3^(-coef$no3)
  # energy conserving reaction, approach I
  Q2 <- sulfate^coef$sulfate * protons^coef$protons2 * S^(-1) * CO2^(-coef$co2)
  # energy conserving reaction, approach II
  Q3 <- sulfate^coef$sulfate * protons^coef$protons3 * NADH^coef$nad * S^(-1) * NAD^(-coef$nad)
  Q4 <- sulfate^coef$sulfate * protons^coef$protons4 * NADH^coef$nad2 * Fdred^coef$fd * 
    FADH2^coef$fad * S^(-1) * NAD^(-coef$nad2) * Fdox^(-coef$fd) * FAD^(-coef$fad)
  # Gibbs free energies, in kJ per mole Sred
  # NO3 reduction to N2
  dGr0_NO3red <- coef$s0*dG0$S0 + coef$sulfate*dG0$sulfate + coef$water1*dG0$water + coef$n2*dG0$N2 +
				coef$protons1*dG0$protons - 1*coef$dG0S - coef$no3*dG0$NO3
  # CO2 reduction  
  if (approach==1) {
    dGr0_CO2fix <- coef$s0*dG0$S0 + coef$sulfate*dG0$sulfate + coef$corg*dG0$Corg + 
				           coef$protons2*dG0$protons + coef$water2*dG0$water - 
			   	         1*coef$dG0S - coef$co2*dG0$CO2 # kJ per mole Sred
    dGr0_RET <- 0
  } 
  else if (approach==2) { 
    if (Cfix=="Calvin") {
      dGr0_CO2fix <- coef$co2*dG0$dG_Calvin # kJ per mole Sred
      dGr0_RET <- coef$s0*dG0$S0 + coef$sulfate*dG0$sulfate + coef$water3*dG0$water +
                  coef$nad*dG0$deltaNAD + coef$protons3*dG0$protons - 1*coef$dG0S # kJ per mole Sred
	  } else if (Cfix=="rTCA") {
	    dGr0_CO2fix <- coef$co2*dG0$dG_rTCA # kJ per mole Sred
      dGr0_RET <- coef$s0*dG0$S0 + coef$sulfate*dG0$sulfate + coef$water3*dG0$water +
                  coef$nad2*dG0$deltaNAD + coef$fad*dG0$deltaFAD+ coef$fd*dG0$deltaFd + 
                  coef$protons4*dG0$protons - 1*coef$dG0S # kJ per mole Sred
	  }    
  }
  dGr0_CO2red <- dGr0_CO2fix + dGr0_RET # kJ per mole Sred
  # calculate T-correction and quotient Q
  if (approach==1) {Qx <- Q2}
  if (approach==2) { 
    if (Cfix=="Calvin") {Qx <- Q3}
    if (Cfix=="rTCA")   {Qx <- Q4}
  }
  corr_NO3red <-  R*(273.15+Temp)*log(Q1)
  corr_CO2red <- R*(273.15+Temp)*log(Qx)
  Q <- Q1^y * Qx^(1-y)
  # apply T-correction
  dGr_NO3red <- dGr0_NO3red + corr_NO3red
  dGr_CO2red <- dGr0_CO2red + corr_CO2red
  # finally calculate the energy conservation efficiencies
  eff <- -(1-y)*dGr_CO2red / ( y*dGr_NO3red )
  # fill also the other output parameters
  if (approach==1) {
	  eff_CO2 <- NA; alpha <- NA; eff_RET_min <- NA; eff_SO_min <- eff
  }
  else if (approach==2) {
	  if (Cfix=="Calvin") {eff_CO2 <- dG0$dG_Calvin/dG0$EATP_Calvin}
	  if (Cfix=="rTCA")   {eff_CO2 <- dG0$dG_rTCA/dG0$EATP_rTCA}
	  alpha <- dGr0_CO2fix/dGr_CO2red
    eff_RET_min <- (1-alpha)/(1/eff - alpha/eff_CO2)
    eff_SO_min <- eff*(alpha/eff_CO2 + (1-alpha))
  }
  # in addition to efficiencies (eff, eff_CO2, eff_RET_min, eff_SO_min), 
  # the function also returns the Gibbs free energies of the energy gaining
  # (dGr_NO3red) and energy conserving (dGr_CO2red) parts of the reaction,
  # as well as the quotient Q and coefficient alpha
  return(list(eff=eff, eff_CO2=eff_CO2, eff_SO_min=eff_SO_min, eff_RET_min=eff_RET_min, 
              alpha=alpha, Q=Q, Q1=Q1, Qx=Qx,
              dGr_RET=dGr0_RET+corr_CO2red, dGr_CO2red=dGr_CO2red, dGr_NO3red=dGr_NO3red))
}

##############################################
# find stoichiometric coefficients for a given
# efficiency and Stot:NO3 ratio
##############################################

# Function for calculating the difference between a given efficiency (eff_fixed) and the efficiency 
# calculated from the stoichiometry parameter x and the Stot:NO3 consumption ratio (input variable S2N).
# This is the function for which a root will need to be found in order to determine stoichiometry
# for a given efficiency and Stot:NO3 consumption ratio.
denit_deff <- function(x=0, S2N=1, eff_fixed=0.05, approach=2, Cfix="Calvin", Sred="H2S", 
			sulfate=1, S=1, NO3=1, N2=1, CO2=1, Temp=25, pH=7, 
			NAD=1, NADH=1, FAD=1, FADH2=1, Fdred=1, Fdox=1){
  if (Sred=="H2S" | Sred=="S2O3") {
	  y <- 1/(S2N*(1.6-1.2*x))
  }
  e <- denit_efficiency(x=x, y=y, approach=approach, Cfix=Cfix, Sred=Sred, 
		  sulfate=sulfate, S=S, NO3=NO3, N2=N2, CO2=CO2, Temp=Temp, pH=pH, 
		  NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox)$eff
  return(e-eff_fixed)
}

# Function for calculating the difference between a given efficiency (eff_fixed) and the efficiency 
# calculated from the stoichiometry parameters x and y.
# This is the function for which a root will need to be found in order to determine stoichiometry
# for a given efficiency and x.
denit_deff_y <- function(y=0.5, x=0, eff_fixed=0.05, approach=2, Cfix="Calvin", Sred="H2S", 
                       sulfate=1, S=1, NO3=1, N2=1, CO2=1, Temp=25, pH=7, 
                       NAD=1, NADH=1, FAD=1, FADH2=1, Fdred=1, Fdox=1){
  e <- denit_efficiency(x=x, y=y, approach=approach, Cfix=Cfix, Sred=Sred, 
                        sulfate=sulfate, S=S, NO3=NO3, N2=N2, CO2=CO2, Temp=Temp, pH=pH, 
                        NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox)$eff
  return(e-eff_fixed)
}

# Function for calculating stoichiometry coefficients (normalized to Sred=1)
# Input: one fixed value of efficiency (eff_fixed), a range of Stot:NO3 ratios (S2N), reactant concentrations
denit_stoichiometry <- function(S2N=1, eff_fixed=0.05, approach=2, Cfix="Calvin", Sred="H2S", 
					sulfate=1, S=1, NO3=1, N2=1, CO2=1, Temp=25, pH=7, 
					NAD=1, NADH=1, FAD=1, FADH2=1, Fdred=1, Fdox=1){
  int <- c(-0.1,1.1)
  #xint <- seq(min(int),max(int),length.out=100) # for debugging
  x <- vector(length=length(S2N))
  y <- vector(length=length(S2N))
  for(i in 1:length(S2N)) {
    s2n <- S2N[i]
    #plot(xint,denit_deff(xint,S2N=s2n,efficiency=eff),type="l") # for debugging
    # find the root of denit_deff, which effectively finds x for which the calculated efficiency equals eff_fixed
    # first check whether it should exist, to avoid error messages of the uniroot function
    if( denit_deff(x=min(int), S2N=s2n, eff_fixed=eff_fixed, approach=approach, Cfix=Cfix, Sred=Sred, 
		    	sulfate=sulfate, S=S, NO3=NO3, N2=N2, CO2=CO2, Temp=Temp, pH=pH,
			    NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox) *
        denit_deff(x=max(int), S2N=s2n, eff_fixed=eff_fixed, approach=approach, Cfix=Cfix, Sred=Sred, 
			    sulfate=sulfate, S=S, NO3=NO3, N2=N2, CO2=CO2, Temp=Temp, pH=pH, 
			    NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox) < 0 ) {
      r <- uniroot(denit_deff, interval=int, S2N=s2n, eff_fixed=eff_fixed, approach=approach, Cfix=Cfix, Sred=Sred, 
			              sulfate=sulfate, S=S, NO3=NO3, N2=N2, CO2=CO2, Temp=Temp, pH=pH, 
			              NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox)
      if( r$root>=-1e-4 & r$root<=1.0001 ) { # store root if it is between 0 and 1 (+/- some tolerance)
        x[i] <- r$root
        #points(x[i],y=0) # for debugging
      } else {
        x[i] <- NA
      }
    } else {
      if( abs(denit_deff(x=0,S2N=s2n, eff_fixed=eff_fixed, approach=approach, Cfix=Cfix, Sred=Sred, 
				        sulfate=sulfate, S=S, NO3=NO3, N2=N2, CO2=CO2, Temp=Temp, pH=pH, 
				        NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox)) < 1e-5 ) {
        x[i] <- 0
      } else {
		    x[i] <- NA
	    }
    }
    #readline("Press enter to continue") # for debugging
	  if (Sred=="H2S" | Sred=="S2O3") {
      y[i] <- 1/(s2n*(1.6-1.2*x[i])) # calculate y from the given s2n and from the found x
	  }
  }
  # select only numbers that are not NA
  i <- !is.na(x)
  x <- x[i]
  y <- y[i]
  S2N <- S2N[i]
  # calculate stoichiometric coefficients for all reactants in the generalized reaction, depending on the kind of Sred
  # note that the coefficient in front of Sred is 1
  coef<-denit_coeffs(x=x, y=y, Sred=Sred)
  # display the stoichiometry of the reaction and the ceresponding energy gained and conserved
  # do this only if *one* value of the Sred:NO3 ratio (S2N) was specified in the input
  if (length(S2N)==1)
  {
    cat(sprintf("Stoichiometry parameters:\tx = %.4f, y = %.4f\n",x,y))
    e <- denit_efficiency(x=x, y=y, approach=approach, Cfix=Cfix, Sred=Sred, 
			sulfate=sulfate, S=S, NO3=NO3, N2=N2,CO2=CO2, Temp=Temp, pH=pH, 
			NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox)
	  cat(sprintf("Energy-gaining reaction:\n\t%.3f %s + %.3f NO3 -> %.3f S0 + %.3f SO4^{2-} + %.3f N2 + %.3f H2O + %.3f H+\n",
                y,Sred,y*coef$no3,y*coef$s0,y*coef$sulfate,y*coef$n2,y*coef$water1,y*coef$protons1))
    cat(sprintf("dG(NO3-reduction) = %.2f kJ/(mol H2S) = %.2f kJ/(mol NO3)\n", e$dGr_NO3red, e$dGr_NO3red/coef$no3))
	  if (approach==1) {
	    cat(sprintf("Energy-conserving reaction:\n\t%.3f %s + %.3f CO2 -> %.3f S0 + %.3f SO4^{2-} + %.3f CH2O + %.3f H2O + %.3f H+\n",
                1-y,Sred,(1-y)*coef$co2,(1-y)*coef$s0,(1-y)*coef$sulfate,(1-y)*coef$co2,(1-y)*coef$water2,(1-y)*coef$protons2))
	  } else if(approach==2) {
		  if(Cfix=="Calvin") {
		    cat(sprintf("Energy-conserving reaction:\n\t%.3f %s + %.3f NAD -> %.3f S0 + %.3f SO4^{2-} + %.3f NADH + %.3f H2O + %.3f H+\n",
              1-y,Sred,(1-y)*coef$nad,(1-y)*coef$s0,(1-y)*coef$sulfate,(1-y)*coef$nad,(1-y)*coef$water3,(1-y)*coef$protons3))
		  }
      if (Cfix=="rTCA") {
		    cat(sprintf("Energy-conserving reaction:\n\t%.3f %s + %.3f NAD + %.3f Fe_ox + %.3f FAD-> %.3f S0 + %.3f SO4^{2-} + %.3f NADH + %.3f Fe_red + %.3f FADH2 + %.3f H2O + %.3f H+\n",
              1-y,Sred,(1-y)*coef$nad2,(1-y)*coef$fd,(1-y)*coef$fad,(1-y)*coef$s0,(1-y)*coef$sulfate,(1-y)*coef$nad2,(1-y)*coef$fd,(1-y)*coef$fad,(1-y)*coef$water3,(1-y)*coef$protons4))
		  }
  	}
    cat(sprintf("dG(CO2-reduction) = %.2f kJ/(mol H2S) = %.2f kJ/(mol CO2)\n", e$dGr_CO2red, e$dGr_CO2red/coef$co2))     
    cat(sprintf("Coupled reaction:\n\t%s + %.3f NO3 + %.3f CO2 -> %.3f S0 + %.3f SO4^{2-} + %.3f N2 + %.3f CH2O + %.3f H2O + %.3f H+\n",
                Sred,coef$nno3,coef$nco2,coef$ns0,coef$nsulfate,coef$nn2,coef$ncorg,coef$nwater,coef$nprotons))    
  }
  return(list(x=x, y=y, NO3=coef$nno3, CO2=coef$nco2, S0=coef$ns0, 
			sulfate=coef$nsulfate, Corg=coef$ncorg, N2=coef$nn2, water=coef$nwater, 
			protons=coef$nprotons, S2N=S2N))
}

# Function for calculating stoichiometry coefficients (normalized to Sred=1)
# Input: one fixed value of efficiency (eff_fixed) and one value of x, reactant concentrations
denit_stoichiometry_y <- function(x=0, eff_fixed=0.05, approach=2, Cfix="Calvin", Sred="H2S", 
                                sulfate=1, S=1, NO3=1, N2=1, CO2=1, Temp=25, pH=7, 
                                NAD=1, NADH=1, FAD=1, FADH2=1, Fdred=1, Fdox=1){
  int <- c(1e-4,1)
  # find the root of denit_deff_y, which effectively finds y for which the calculated efficiency equals eff_fixed
  # first check whether it should exist, to avoid error messages of the uniroot function
  if( denit_deff_y(y=min(int), x=x, eff_fixed=eff_fixed, approach=approach, Cfix=Cfix, Sred=Sred, 
                   sulfate=sulfate, S=S, NO3=NO3, N2=N2, CO2=CO2, Temp=Temp, pH=pH,
                   NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox) *
      denit_deff_y(y=max(int), x=x, eff_fixed=eff_fixed, approach=approach, Cfix=Cfix, Sred=Sred, 
                   sulfate=sulfate, S=S, NO3=NO3, N2=N2, CO2=CO2, Temp=Temp, pH=pH, 
                   NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox) < 0 ) {
    r <- uniroot(denit_deff_y, interval=int, x=x, eff_fixed=eff_fixed, approach=approach, Cfix=Cfix, Sred=Sred, 
                 sulfate=sulfate, S=S, NO3=NO3, N2=N2, CO2=CO2, Temp=Temp, pH=pH, 
                 NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox)
    if( r$root>=-1e-4 & r$root<=1.0001 ) { # store root if it is between 0 and 1 (+/- some tolerance)
      y <- r$root
    } else {
      y <- NA
    }
  } else {
    if( abs(denit_deff_y(y=0,x=x, eff_fixed=eff_fixed, approach=approach, Cfix=Cfix, Sred=Sred, 
                       sulfate=sulfate, S=S, NO3=NO3, N2=N2, CO2=CO2, Temp=Temp, pH=pH, 
                       NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox)) < 1e-5 ) {
      y <- 0
    } else {
      y <- NA
    }
  }
  # calculate stoichiometric coefficients for all reactants in the generalized reaction, depending on the kind of Sred
  # note that the coefficient in front of Sred is 1
  coef<-denit_coeffs(x=x, y=y, Sred=Sred)
  # display the stoichiometry of the reaction and the ceresponding energy gained and conserved
  # do this only if *one* value of the Sred:NO3 ratio (S2N) was specified in the input
  if (!is.na(y))
  {
    cat(sprintf("Stoichiometry parameters:\tx = %.4f, y = %.4f\n",x,y))
    e <- denit_efficiency(x=x, y=y, approach=approach, Cfix=Cfix, Sred=Sred, 
                          sulfate=sulfate, S=S, NO3=NO3, N2=N2, CO2=CO2, Temp=Temp, pH=pH, 
                          NAD=NAD, NADH=NADH, FAD=FAD, FADH2=FADH2, Fdred=Fdred, Fdox=Fdox)
    cat(sprintf("Energy-gaining reaction:\n\t%.3f %s + %.3f NO3 -> %.3f S0 + %.3f SO4^{2-}+ %.3f N2 + %.3f H2O + %.3f H+\n",
                y,Sred,y*coef$no3,y*coef$s0,y*coef$sulfate,y*coef$n2,y*coef$water1,y*coef$protons1))
    cat(sprintf("dG(NO3-reduction) = %.2f kJ/(mol H2S) = %.2f kJ/(mol NO3)\n", e$dGr_NO3red, e$dGr_NO3red/coef$no3))
    if (approach==1) {
      cat(sprintf("Energy-conserving reaction:\n\t%.3f %s + %.3f CO2 -> %.3f S0 + %.3f SO4^{2-} + %.3f CH2O + %.3f H2O + %.3f H+\n",
                  1-y,Sred,(1-y)*coef$co2,(1-y)*coef$s0,(1-y)*coef$sulfate,(1-y)*coef$co2,(1-y)*coef$water2,(1-y)*coef$protons2))
    } else if(approach==2) {
      if(Cfix=="Calvin") {
        cat(sprintf("Energy-conserving reaction:\n\t%.3f %s + %.3f NAD -> %.3f S0 + %.3f SO4^{2-} + %.3f NADH + %.3f H2O + %.3f H+\n",
                    1-y,Sred,(1-y)*coef$nad,(1-y)*coef$s0,(1-y)*coef$sulfate,(1-y)*coef$nad,(1-y)*coef$water3,(1-y)*coef$protons3))
      }
      if (Cfix=="rTCA") {
        cat(sprintf("Energy-conserving reaction:\n\t%.3f %s + %.3f NAD + %.3f Fe_ox + %.3f FAD-> %.3f S0 + %.3f SO4^{2-} + %.3f NADH + %.3f Fe_red + %.3f FADH2 + %.3f H2O + %.3f H+\n",
                    1-y,Sred,(1-y)*coef$nad2,(1-y)*coef$fd,(1-y)*coef$fad,(1-y)*coef$s0,(1-y)*coef$sulfate,(1-y)*coef$nad2,(1-y)*coef$fd,(1-y)*coef$fad,(1-y)*coef$water3,(1-y)*coef$protons4))
      }
    }
    cat(sprintf("dG(CO2-reduction) = %.2f kJ/(mol H2S) = %.2f kJ/(mol CO2)\n", e$dGr_CO2red, e$dGr_CO2red/coef$co2))     
    cat(sprintf("Coupled reaction:\n\t%s + %.3f NO3 + %.3f CO2 -> %.3f S0 + %.3f SO4^{2-} + %.3f N2 + %.3f CH2O + %.3f H2O + %.3f H+\n",
                Sred,coef$nno3,coef$nco2,coef$ns0,coef$nsulfate,coef$nn2,coef$ncorg,coef$nwater,coef$nprotons))    
  }
  return(list(x=x, y=y, NO3=coef$nno3, CO2=coef$nco2, S0=coef$ns0, 
              sulfate=coef$nsulfate, Corg=coef$ncorg, N2=coef$nn2, water=coef$nwater, 
              protons=coef$nprotons))
}

