# Examples demonstrating the use of the formalism described by Klatt & Polerecky (2015) 
# doi: 10.3389/fmicb.2015.00484

# Calculate the efficiency as a function of x, assuming that the CO2:TEA ratio is constant. 
# Perform for aerobic sulfide oxidation and anaerobic thiosulfate oxidation coupled to denitrification. 
# Calculate at standard biochemical conditions as well as at reactant concentrations of 1mM.

# to load the functions
source('SOX_functions.R')

# aerobic sulfide oxidation

# Beggiatoa str. MS-81-6
y <- 0.825 # this value of y essentially determines the CO2:TEA ratio
x <- seq(from=0,to=1,length.out=100) 
(e1 <- aerob_efficiency(x,y, Sred="H2S", approach=2, Cfix="Calvin")$eff)
(e2 <- aerob_efficiency(x,y, Sred="H2S", approach=2, Cfix="Calvin", 
                        sulfate=1e-3, S=1e-3, O2=1e-3, CO2=1e-3, Temp=25, pH=7, 
                        NAD=1e-3, NADH=1e-3, FAD=1e-3, FADH2=1e-3, Fdred=1e-3, Fdox=1e-3)$eff)

# Beggiatoa str. MS-81-1c
y <- 0.641
(e3 <- aerob_efficiency(x,y, Sred="H2S", approach=2, Cfix="Calvin")$eff)
(e4 <- aerob_efficiency(x,y, Sred="H2S", approach=2, Cfix="Calvin", 
                        sulfate=1e-3, S=1e-3, O2=1e-3, CO2=1e-3, Temp=25, pH=7, 
                        NAD=1e-3, NADH=1e-3, FAD=1e-3, FADH2=1e-3, Fdred=1e-3, Fdox=1e-3)$eff)

# plot the efficiency vs. x
x11(type="nbcairo")
plot(x,e1,type="l",lwd=2,lty=1,col='black',
     xlim=c(0,1),
     ylim=c(0.0,0.5),
     xlab="x",ylab=expression(epsilon[II]),cex.lab=1.4,cex.axis=1.4)
lines(x,e2,lwd=2,lty=2,col='black')
lines(x,e3,lwd=2,lty=1,col='gray50')
lines(x,e4,lwd=2,lty=2,col='gray50')

# include legend
legend('topright',legend=c("std. conditions","1 mM, pH 7"),
		bty="n", pch=c(NA,NA), lty=c(1,2), 
		lwd=c(2,2),cex=1.4)

# incude annotation
text(x=0.01,y=0.3,"Beggiatoa str. MS-81-1c",col="gray50",adj=c(0,NA),cex=1.2)
text(x=0.01,y=0.125,"Beggiatoa str. MS-81-6",col="black",adj=c(0,NA),cex=1.2)
text(x=0.01,y=0.01,"CO2:O2 = constant",col="black",adj=c(0,NA),cex=1.2)

# export to EPS
dev.copy2eps(file='efficiency-x-aerobic-A.eps')



# anaerobic thiosulfate oxidation coupled to denitrification

# Sulfurimonas denitrificans
y <- 0.902 # this value of y essentially determines the CO2:TEA ratio
x <- seq(from=0,to=1,length.out=100) 
(e1 <- denit_efficiency(x,y, Sred="S2O3", approach=2, Cfix="rTCA")$eff)
(e2 <- denit_efficiency(x,y, Sred="S2O3", approach=2, Cfix="rTCA", 
                        sulfate=1e-3, S=1e-3, NO3=1e-3, N2=1e-3, CO2=1e-3, Temp=25, pH=7, 
                        NAD=1e-3, NADH=1e-3, FAD=1e-3, FADH2=1e-3, Fdred=1e-3, Fdox=1e-3)$eff)

# Sulfurimonas hongkongensis
y <- 0.852
(e3 <- denit_efficiency(x,y, Sred="S2O3", approach=2, Cfix="rTCA")$eff)
(e4 <- denit_efficiency(x,y, Sred="S2O3", approach=2, Cfix="rTCA", 
                        sulfate=1e-3, S=1e-3, NO3=1e-3, N2=1e-3, CO2=1e-3, Temp=25, pH=7, 
                        NAD=1e-3, NADH=1e-3, FAD=1e-3, FADH2=1e-3, Fdred=1e-3, Fdox=1e-3)$eff)

# plot the efficiency vs. x
x11(type="nbcairo")
plot(x,e1,type="l",lwd=2,lty=1,col='black',
     xlim=c(0,1),
     ylim=c(0.0,0.1),
     xlab="x",ylab=expression(epsilon[II]),cex.lab=1.4,cex.axis=1.4)
lines(x,e2,lwd=2,lty=2,col='black')
lines(x,e3,lwd=2,lty=1,col='gray50')
lines(x,e4,lwd=2,lty=2,col='gray50')

# include legend
legend('topright',legend=c("std. conditions","1 mM, pH 7"),
       bty="n", pch=c(NA,NA), lty=c(1,2), 
       lwd=c(2,2),cex=1.4)

# incude annotation
text(x=0.01,y=0.082,"Sulfurimonas hongkongensis",col="gray50",adj=c(0,NA),cex=1.2)
text(x=0.01,y=0.038,"Sulfurimonas denitrificans",col="black",adj=c(0,NA),cex=1.2)
text(x=0.01,y=0.003,"CO2:NO3 = constant",col="black",adj=c(0,NA),cex=1.2)

# export to EPS
dev.copy2eps(file='efficiency-x-denit-A.eps')
