# Examples demonstrating the use of the formalism described by Klatt & Polerecky (2015) 
# doi: 10.3389/fmicb.2015.00484

# Perform sensitivity analysis of the efficiency calculation

# load the functions
source('SOX_functions.R')

# aerobic thiosulfate oxidation by Thermithiobacillus tepidarius:
# data from Wood and Kelly (1986) imply x=0 and y=0.804, C-fixation through Calvin cycle;
x <- 0; y <- 0.804

# set the range of reactant concentrations from 1uM to 1M
crange <- 10^seq(-9,0,1)

# calculate the efficiencies, traditional approach
e0 <- aerob_efficiency(x,y,Sred="S2O3",approach=1,pH=7) # std biochemical conditions
e1 <- aerob_efficiency(x,y,Sred="S2O3",approach=1,sulfate=crange,pH=7) # vary sulfate, other conditions standard biochemical
e2 <- aerob_efficiency(x,y,Sred="S2O3",approach=1,S=crange,pH=7) # vary S=S2O3, other conditions standard biochemical
e3 <- aerob_efficiency(x,y,Sred="S2O3",approach=1,O2=crange,pH=7) # vary O2, other conditions standard biochemical
e4 <- aerob_efficiency(x,y,Sred="S2O3",approach=1,CO2=crange,pH=7) # vary CO2, other conditions standard biochemical
e5 <- aerob_efficiency(x,y,Sred="S2O3",approach=1,pH=(2:14)) # vary pH from 2 to 14, other conditions standard biochemical

# calculate the efficiencies, new approach
d0 <- aerob_efficiency(x,y,Sred="S2O3",approach=2,Cfix="Calvin",pH=7) # std biochemical conditions
d1 <- aerob_efficiency(x,y,Sred="S2O3",approach=2,Cfix="Calvin",sulfate=crange,pH=7) # vary sulfate, other conditions standard physiological
d2 <- aerob_efficiency(x,y,Sred="S2O3",approach=2,Cfix="Calvin",S=crange,pH=7) # vary S=S2O3, other conditions standard physiological
d3 <- aerob_efficiency(x,y,Sred="S2O3",approach=2,Cfix="Calvin",O2=crange,pH=7) # vary O2, other conditions standard physiological
d4 <- aerob_efficiency(x,y,Sred="S2O3",approach=2,Cfix="Calvin",NAD=crange,pH=7) # vary NADP, other conditions standard physiological
d5 <- aerob_efficiency(x,y,Sred="S2O3",approach=2,Cfix="Calvin",pH=(2:14)) # vary pH, other conditions standard physiological
d6 <- aerob_efficiency(x,y,Sred="S2O3",approach=2,Cfix="Calvin",NADH=crange,pH=7) # vary NADPH, other conditions standard physiological

# aerobic thiosulfate oxidation by Beggiatoa str. D-402:
# data from Grabovich et al. (2001) imply x=0 and y=0.761, C-fixation through Calvin cycle;
x <- 0; y <- 0.761

# set the range of reactant concentrations from 1uM to 1M
crange <- 10^seq(-9,0,1) 

# calculate the efficiencies, traditional approach
a0 <- aerob_efficiency(x,y,Sred="S2O3",approach=1,pH=7) # std biochemical conditions
a1 <- aerob_efficiency(x,y,Sred="S2O3",approach=1,sulfate=crange,pH=7) # vary sulfate, other conditions standard biochemical
a2 <- aerob_efficiency(x,y,Sred="S2O3",approach=1,S=crange,pH=7) # vary S=S2O3, other conditions standard biochemical
a3 <- aerob_efficiency(x,y,Sred="S2O3",approach=1,O2=crange,pH=7) # vary O2, other conditions standard biochemical
a4 <- aerob_efficiency(x,y,Sred="S2O3",approach=1,CO2=crange,pH=7) # vary CO2, other conditions standard biochemical
a5 <- aerob_efficiency(x,y,Sred="S2O3",approach=1,pH=(2:14)) # vary pH, other conditions standard biochemical

# calculate the efficiencies, new approach
b0 <- aerob_efficiency(x,y,Sred="S2O3",approach=2,Cfix="Calvin",pH=7) # std biochemical conditions
b1 <- aerob_efficiency(x,y,Sred="S2O3",approach=2,Cfix="Calvin",sulfate=crange,pH=7) # vary sulfate, other conditions standard physiological
b2 <- aerob_efficiency(x,y,Sred="S2O3",approach=2,Cfix="Calvin",S=crange,pH=7) # vary S=S2O3, other conditions standard physiological
b3 <- aerob_efficiency(x,y,Sred="S2O3",approach=2,Cfix="Calvin",O2=crange,pH=7) # vary O2, other conditions standard physiological
b4 <- aerob_efficiency(x,y,Sred="S2O3",approach=2,Cfix="Calvin",NAD=crange,pH=7) # vary NADP, other conditions standard physiological
b5 <- aerob_efficiency(x,y,Sred="S2O3",approach=2,Cfix="Calvin",pH=(2:14)) # vary pH from 2 to 10, other conditions standard physiological
b6 <- aerob_efficiency(x,y,Sred="S2O3",approach=2,Cfix="Calvin",NADH=crange,pH=7) # vary NADPH, other conditions standard physiological

# store data, will be used to determine scaling
xs <- c(e1$Q,e2$Q,e3$Q,e4$Q,e5$Q,d1$Q,d2$Q,d3$Q,d4$Q,d5$Q,d6$Q,
        a1$Q,a2$Q,a3$Q,a4$Q,a5$Q,b1$Q,b2$Q,b3$Q,b4$Q,b5$Q,b6$Q)
ys <- c(e1$eff_SO_min,e2$eff_SO_min,e3$eff_SO_min,e4$eff_SO_min,e5$eff_SO_min,d1$eff_SO_min,d2$eff_SO_min,d3$eff_SO_min,d4$eff_SO_min,d5$eff_SO_min,d6$eff_SO_min,
        a1$eff_SO_min,a2$eff_SO_min,a3$eff_SO_min,a4$eff_SO_min,a5$eff_SO_min,b1$eff_SO_min,b2$eff_SO_min,b3$eff_SO_min,b4$eff_SO_min,b5$eff_SO_min,b6$eff_SO_min)

# plot the efficiencies as a function of Q, and export it
x11(type="nbcairo")

# plot the data for Thermithiobacillus tepidarius in gray
plot(e1$Q,e1$eff_SO_min,type="l",lwd=2,lty=1,col='gray50',log="x",
     #xlim=c(1e-20,1e10),
     xlim=c(min(xs),max(xs)),
     #ylim=c(0.01,0.1),
     ylim=c(min(ys),max(ys)),
     xlab="Q",ylab=expression(epsilon),cex.lab=1.4,cex.axis=1.4)
lines(e2$Q,e2$eff_SO_min,lty=2,lwd=2,col='gray50')
lines(e3$Q,e3$eff_SO_min,lty=4,lwd=2,col='gray50')
lines(e4$Q,e4$eff_SO_min,lty=3,lwd=2,col='gray50')
points(e5$Q,e5$eff_SO_min,pch=1,col='gray50')
points(e0$Q,e0$eff_SO_min,pch=19,col='gray50',cex=2)
lines(d1$Q,d1$eff_SO_min,lty=1,lwd=2,col='gray50')
lines(d2$Q,d2$eff_SO_min,lty=2,lwd=2,col='gray50')
lines(d3$Q,d3$eff_SO_min,lty=4,lwd=2,col='gray50')
points(d5$Q,d5$eff_SO_min,pch=1,col='gray50')
points(d4$Q,d4$eff_SO_min,pch=0,col='gray50')
points(d6$Q,d6$eff_SO_min,pch=2,col='gray50')
points(d0$Q,d0$eff_SO_min,pch=19,col='gray50',cex=2)

# plot the data for Beggiatoa str. D-402 in black
lines(a1$Q,a1$eff_SO_min,lty=1,lwd=2,col='black')
lines(a2$Q,a2$eff_SO_min,lty=2,lwd=2,col='black')
lines(a3$Q,a3$eff_SO_min,lty=4,lwd=2,col='black')
lines(a4$Q,a4$eff_SO_min,lty=3,lwd=2,col='black')
points(a5$Q,a5$eff_SO_min,pch=1,col='black')
points(a0$Q,a0$eff_SO_min,pch=19,col='black',cex=2)
lines(b1$Q,b1$eff_SO_min,lty=1,lwd=2,col='black')
lines(b2$Q,b2$eff_SO_min,lty=2,lwd=2,col='black')
lines(b3$Q,b3$eff_SO_min,lty=4,lwd=2,col='black')
points(b5$Q,b5$eff_SO_min,pch=1,col='black')
points(b4$Q,b4$eff_SO_min,pch=0,col='black')
points(b6$Q,b6$eff_SO_min,pch=2,col='black')
points(b0$Q,b0$eff_SO_min,pch=19,col='black',cex=2)

# include legend
legend('topleft',legend=c(expression(SO[4]),expression(S[2]*O[3]),
                          expression(O[2]),expression(CO[2]),"pH",expression(NAD^"+"),"NADH"),
       bty="n", pch=c(NA,NA,NA,NA,1,0,2), lty=c(1,2,4,3,NA,NA,NA), 
       lwd=c(2,2,2,2,NA,NA,NA),cex=1)

# incude annotation
text(x=1e-15,y=0.023,expression(epsilon[I]),col="gray50",adj=c(0,NA),cex=1.4)
text(x=1e-14,y=0.065,expression(epsilon[I]),col="black",adj=c(1,NA),cex=1.4)
text(x=1e-27,y=0.19,expression(epsilon[SO][II][min]),col="black",adj=c(0,NA),cex=1.4)
text(x=1e-13,y=0.122,expression(epsilon[SO][II][min]),col="gray50",adj=c(1,NA),cex=1.4)

# export to EPS
dev.copy2eps(file='eff_sensitivity_aerobic.eps')




# anaerobic thiosulfate oxidation coupled to denitrification by Sulfurimonas denitrificans 
# C-fixation through rTCA cycle
x <- 0; y <- 0.902

# set the range of reactant concentrations from 1uM to 1M
crange <- 10^seq(-9,0,1)

# calculate the efficiencies, traditional approach
e0 <- denit_efficiency(x,y,Sred="S2O3",approach=1,pH=7) # std biochemical conditions
e1 <- denit_efficiency(x,y,Sred="S2O3",approach=1,sulfate=crange,pH=7) # vary sulfate, other conditions standard biochemical
e2 <- denit_efficiency(x,y,Sred="S2O3",approach=1,S=crange,pH=7) # vary S=S2O3, other conditions standard biochemical
e3 <- denit_efficiency(x,y,Sred="S2O3",approach=1,NO3=crange,pH=7) # vary NO3, other conditions standard biochemical
e4 <- denit_efficiency(x,y,Sred="S2O3",approach=1,CO2=crange,pH=7) # vary CO2, other conditions standard biochemical
e5 <- denit_efficiency(x,y,Sred="S2O3",approach=1,pH=(2:14)) # vary pH, other conditions standard biochemical
e6 <- denit_efficiency(x,y,Sred="S2O3",approach=1,N2=crange,pH=7) # vary N2, other conditions standard biochemical

# calculate the efficiencies, new approach
d0 <- denit_efficiency(x,y,Sred="S2O3",approach=2,Cfix="rTCA",pH=7) # std biochemical conditions
d1 <- denit_efficiency(x,y,Sred="S2O3",approach=2,Cfix="rTCA",sulfate=crange,pH=7) # vary sulfate, other conditions standard physiological
d2 <- denit_efficiency(x,y,Sred="S2O3",approach=2,Cfix="rTCA",S=crange,pH=7) # vary S=S2O3, other conditions standard physiological
d3 <- denit_efficiency(x,y,Sred="S2O3",approach=2,Cfix="rTCA",NO3=crange,pH=7) # vary NO3, other conditions standard physiological
d4 <- denit_efficiency(x,y,Sred="S2O3",approach=2,Cfix="rTCA",NAD=crange,pH=7) # vary NAD, other conditions standard physiological
d5 <- denit_efficiency(x,y,Sred="S2O3",approach=2,Cfix="rTCA",pH=(2:14)) # vary pH, other conditions standard physiological
d6 <- denit_efficiency(x,y,Sred="S2O3",approach=2,Cfix="rTCA",NADH=crange,pH=7) # vary NADH, other conditions standard physiological
d7 <- denit_efficiency(x,y,Sred="S2O3",approach=2,Cfix="rTCA",N2=crange,pH=7) # vary N2, other conditions standard biochemical

# anaerobic thiosulfate oxidation coupled to denitrification by Sulfurimonas hongkongensis
# C-fixation through rTCA cycle
x <- 0; y <- 0.852

# set the range of reactant concentrations from 1uM to 1M
crange <- 10^seq(-9,0,1) 

# calculate the efficiencies, approach I
a0 <- denit_efficiency(x,y,Sred="S2O3",approach=1,pH=7) # std biochemical conditions
a1 <- denit_efficiency(x,y,Sred="S2O3",approach=1,sulfate=crange,pH=7) # vary sulfate, other conditions standard biochemical
a2 <- denit_efficiency(x,y,Sred="S2O3",approach=1,S=crange,pH=7) # vary S=S2O3, other conditions standard biochemical
a3 <- denit_efficiency(x,y,Sred="S2O3",approach=1,NO3=crange,pH=7) # vary NO3, other conditions standard biochemical
a4 <- denit_efficiency(x,y,Sred="S2O3",approach=1,CO2=crange,pH=7) # vary CO2, other conditions standard biochemical
a5 <- denit_efficiency(x,y,Sred="S2O3",approach=1,pH=(2:14)) # vary pH, other conditions standard biochemical
a6 <- denit_efficiency(x,y,Sred="S2O3",approach=1,N2=crange,pH=7) # vary N2, other conditions standard biochemical

# calculate the efficiencies, approach II
b0 <- denit_efficiency(x,y,Sred="S2O3",approach=2,Cfix="rTCA",pH=7) # std biochemical conditions
b1 <- denit_efficiency(x,y,Sred="S2O3",approach=2,Cfix="rTCA",sulfate=crange,pH=7) # vary sulfate, other conditions standard physiological
b2 <- denit_efficiency(x,y,Sred="S2O3",approach=2,Cfix="rTCA",S=crange,pH=7) # vary S=S2O3, other conditions standard physiological
b3 <- denit_efficiency(x,y,Sred="S2O3",approach=2,Cfix="rTCA",NO3=crange,pH=7) # vary NO3, other conditions standard physiological
b4 <- denit_efficiency(x,y,Sred="S2O3",approach=2,Cfix="rTCA",NAD=crange,pH=7) # vary NAD, other conditions standard physiological
b5 <- denit_efficiency(x,y,Sred="S2O3",approach=2,Cfix="rTCA",pH=(2:14)) # vary pH, other conditions standard physiological
b6 <- denit_efficiency(x,y,Sred="S2O3",approach=2,Cfix="rTCA",NADH=crange,pH=7) # vary NADH, other conditions standard physiological
b7 <- denit_efficiency(x,y,Sred="S2O3",approach=2,Cfix="rTCA",N2=crange,pH=7) # vary N2, other conditions standard biochemical

# store data, will be used to determine scaling
xs <- c(e1$Q,e2$Q,e3$Q,e4$Q,e5$Q,d1$Q,d2$Q,d3$Q,d4$Q,d5$Q,d6$Q,
        a1$Q,a2$Q,a3$Q,a4$Q,a5$Q,b1$Q,b2$Q,b3$Q,b4$Q,b5$Q,b6$Q)
ys <- c(e1$eff_SO_min,e2$eff_SO_min,e3$eff_SO_min,e4$eff_SO_min,e5$eff_SO_min,d1$eff_SO_min,d2$eff_SO_min,d3$eff_SO_min,d4$eff_SO_min,d5$eff_SO_min,d6$eff_SO_min,
        a1$eff_SO_min,a2$eff_SO_min,a3$eff_SO_min,a4$eff_SO_min,a5$eff_SO_min,b1$eff_SO_min,b2$eff_SO_min,b3$eff_SO_min,b4$eff_SO_min,b5$eff_SO_min,b6$eff_SO_min)

# plot the efficiencies as a function of Q, and export it
x11(type="nbcairo")

# plot the data for Sulfurimonas denitrificans in gray
plot(e1$Q,e1$eff_SO_min,type="l",lwd=2,lty=1,col='gray50',log="x",
     #xlim=c(1e-20,1e10),
     xlim=c(min(xs),max(xs)),
     #ylim=c(0.01,0.1),
     ylim=c(min(ys),max(ys)),
     xlab="Q",ylab=expression(epsilon),cex.lab=1.4,cex.axis=1.4)
lines(e2$Q,e2$eff_SO_min,lty=2,lwd=2,col='gray50')
lines(e3$Q,e3$eff_SO_min,lty=4,lwd=2,col='gray50')
lines(e4$Q,e4$eff_SO_min,lty=3,lwd=2,col='gray50')
points(e5$Q,e5$eff_SO_min,pch=1,col='gray50')
points(e6$Q,e6$eff_SO_min,pch=3,col='gray50')
points(e0$Q,e0$eff_SO_min,pch=19,col='gray50',cex=2)
lines(d1$Q,d1$eff_SO_min,lty=1,lwd=2,col='gray50')
lines(d2$Q,d2$eff_SO_min,lty=2,lwd=2,col='gray50')
lines(d3$Q,d3$eff_SO_min,lty=4,lwd=2,col='gray50')
points(d5$Q,d5$eff_SO_min,pch=1,col='gray50')
points(d7$Q,d7$eff_SO_min,pch=3,col='gray50')
points(d4$Q,d4$eff_SO_min,pch=0,col='gray50')
points(d6$Q,d6$eff_SO_min,pch=2,col='gray50')
points(d0$Q,d0$eff_SO_min,pch=19,col='gray50',cex=2)

# plot the data for Sulfurimonas hongkongensis in black
lines(a1$Q,a1$eff_SO_min,lty=1,lwd=2,col='black')
lines(a2$Q,a2$eff_SO_min,lty=2,lwd=2,col='black')
lines(a3$Q,a3$eff_SO_min,lty=4,lwd=2,col='black')
lines(a4$Q,a4$eff_SO_min,lty=3,lwd=2,col='black')
points(a5$Q,a5$eff_SO_min,pch=1,col='black')
points(a6$Q,a6$eff_SO_min,pch=3,col='black')
points(a0$Q,a0$eff_SO_min,pch=19,col='black',cex=2)
lines(b1$Q,b1$eff_SO_min,lty=1,lwd=2,col='black')
lines(b2$Q,b2$eff_SO_min,lty=2,lwd=2,col='black')
lines(b3$Q,b3$eff_SO_min,lty=4,lwd=2,col='black')
points(b5$Q,b5$eff_SO_min,pch=1,col='black')
points(b7$Q,b7$eff_SO_min,pch=3,col='black')
points(b4$Q,b4$eff_SO_min,pch=0,col='black')
points(b6$Q,b6$eff_SO_min,pch=2,col='black')
points(b0$Q,b0$eff_SO_min,pch=19,col='black',cex=2)

# include legend
legend('topleft',legend=c(expression(SO[4]),expression(S[2]*O[3]),
                          expression(NO[3]),expression(CO[2]),"pH",expression(N[2]),expression(NAD^"+"),"NADH"),
       bty="n", pch=c(NA,NA,NA,NA,1,3,0,2), lty=c(1,2,4,3,NA,NA,NA,NA), 
       lwd=c(2,2,2,2,NA,NA,NA,NA),cex=1)

# incude annotation
text(x=1e-4,y=0.012,expression(epsilon[I]),col="gray50",adj=c(0,NA),cex=1.4)
text(x=1e-5,y=0.037,expression(epsilon[I]),col="black",adj=c(1,NA),cex=1.4)
text(x=1e-13,y=0.056,expression(epsilon[SO][II][min]),col="gray50",adj=c(0,NA),cex=1.4)
text(x=1e-10,y=0.085,expression(epsilon[SO][II][min]),col="black",adj=c(1,NA),cex=1.4)

# export to EPS
dev.copy2eps(file='eff_sensitivity_denit.eps')

