# Examples demonstrating the use of the formalism described by Klatt & Polerecky (2015) 
# doi: 10.3389/fmicb.2015.00484

# Calculate stoichiometry of autotrophic sulfur oxidation from the known efficiency and Sred:TEA ratio
# TEA = terminal electron acceptor (O2 or NO3)
# Sred = reduced sulfur compound (H2S or S2O3)

# load the functions
source('SOX_functions.R')

#######################################
# Aerobic autotrophic sulfur oxidation
#######################################

# aerobic sulfide oxidation, traditional approach, standard biochemical conditions
s1 <- aerob_stoichiometry(eff_fixed=0.04, S2O=2, Sred="H2S", approach=1)

# aerobic sulfide oxidation, new approach, standard biochemical conditions, CO2 fixation via Calvin cycle
s2 <- aerob_stoichiometry(eff_fixed=0.04, S2O=2, Sred="H2S", approach=2, Cfix="Calvin")
# aerobic sulfide oxidation, new approach, standard biochemical conditions, CO2 fixation via rTCA cycle
s2b <- aerob_stoichiometry(eff_fixed=0.04, S2O=2, Sred="H2S", approach=2, Cfix="rTCA")

# efficiency and stoichiometry of aerobic thiosulfate oxidation by Thioalkalivibrio versutus: 
# data from Sorokin et al. (2001) imply x=0 and y=0.844, C-fixation through Calvin cycle; 
x <- 0; y <- 0.844
# efficiency at std biochemical conditions, traditional and new approach
(e3<-aerob_efficiency(x,y,Sred="S2O3",approach=1)$eff)
(e4<-aerob_efficiency(x,y,Sred="S2O3",approach=2,Cfix="Calvin")$eff)
# the corresponding stoichiometry (should be the same for both approaches)
# note that for aerobic sulfur oxidation, the S2O ratio is related to x and y through a different expression then
# for anaerobic sulfur oxidation! (see Supplementary tables S1)
s3 <- aerob_stoichiometry(eff_fixed=e3, S2O=1/(y*(2-1.5*x)), Sred="S2O3", approach=1)
s4 <- aerob_stoichiometry(eff_fixed=e4, S2O=1/(y*(2-1.5*x)), Sred="S2O3", approach=2, Cfix="Calvin")

# efficiency and stoichiometry of aerobic thiosulfate oxidation by Thermithiobacillus tepidarius:
# data from Wood and Kelly (1986) imply x=0 and y=0.804, C-fixation through Calvin cycle;
x <- 0; y <- 0.804
# efficiency at std biochemical conditions, traditional and new approach
(e5 <- aerob_efficiency(x,y,Sred="S2O3",approach=1)$eff)
(e6 <- aerob_efficiency(x,y,Sred="S2O3",approach=2,Cfix="Calvin")$eff)
# note that for aerobic sulfur oxidation, the S2O ratio is related to x and y through a different expression then
# for anaerobic sulfur oxidation! (see Supplementary tables S1)
s5 <- aerob_stoichiometry(eff_fixed=e5, S2O=1/(y*(2-1.5*x)), Sred="S2O3", approach=1)
s6 <- aerob_stoichiometry(eff_fixed=e6, S2O=1/(y*(2-1.5*x)), Sred="S2O3", approach=2, Cfix="Calvin")

###########################################################
# Autotrophic sulfur oxidation coupled to denitrification
###########################################################

# anaerobic thiosulfate oxidation, traditional approach, standard biochemical conditions, CO2 fixation via rTCA cycle
s3 <- denit_stoichiometry(eff_fixed=0.04, S2N=2, Sred="S2O3", approach=2, Cfix="rTCA")

# efficiency and stoichiometry of anaerobic thiosulfate oxidation coupled to denitrifiction by Sulfurimonas denitrificans: 
# data from Hoor (1981) imply x=0 and y=0.835, C-fixation through rTCA cycle;  
x <- 0; y <- 0.835
# efficiency at std conditions, traditional and new approach
(e1<-denit_efficiency(x,y,Sred="S2O3",approach=1)$eff)
(e2<-denit_efficiency(x,y,Sred="S2O3",approach=2,Cfix="rTCA")$eff)
# the corresponding stoichiometry (should be the same for both approaches)
s1 <- denit_stoichiometry(eff_fixed=e1, S2N=1/(y*(1.6-1.2*x)), Sred="S2O3", approach=1)
s2 <- denit_stoichiometry(eff_fixed=e2, S2N=1/(y*(1.6-1.2*x)), Sred="S2O3", approach=2, Cfix="rTCA")
